/* --------- XFGL - OpenGL Lighting / XForms Control --------- */
/* (C) 960315 -- Anneke Sicherer-Roetman -- anneke@chem.ruu.nl */
/* Note: if you find this code useful, please drop me an email */

#include <forms.h>
#include <GL/glut.h>
#include "xfgl1form.h"

static GLfloat ambient[3];  
static GLfloat diffuse[3];  
static GLfloat specular[3];
static GLfloat position[4];
static GLfloat shine;
static int viewportWidth,viewportHeight;

static void UpdateLighting(void)
{
   glMaterialfv(GL_FRONT,GL_AMBIENT,ambient);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,diffuse);
   glMaterialfv(GL_FRONT,GL_SPECULAR,specular);
   glMaterialf(GL_FRONT,GL_SHININESS,shine*128);
   glLightfv(GL_LIGHT0,GL_POSITION,position);
}

static void UpdateSliders(void)
{
   fl_set_slider_value(ambientR,ambient[0]);
   fl_set_slider_value(ambientG,ambient[1]);
   fl_set_slider_value(ambientB,ambient[2]);
   fl_set_slider_value(diffuseR,diffuse[0]);
   fl_set_slider_value(diffuseG,diffuse[1]);
   fl_set_slider_value(diffuseB,diffuse[2]);
   fl_set_slider_value(specularR,specular[0]);
   fl_set_slider_value(specularG,specular[1]);
   fl_set_slider_value(specularB,specular[2]);
   fl_set_slider_value(positionR,position[0]);
   fl_set_slider_value(positionG,position[1]);
   fl_set_slider_value(positionB,position[2]);
   fl_set_slider_value(shininess,shine);
}

void Lighting(FL_OBJECT *ob, long data) /* XForms callback */
{
   switch (data) {
   case  0: ambient[0]=ambient[1]=ambient[2]=0.5;
            diffuse[0]=diffuse[1]=diffuse[2]=0.5;
            specular[0]=specular[1]=specular[2]=0.5;
            position[0]=position[1]=position[2]=0.5; 
            position[4]=0.0; shine=0.5; 
            UpdateSliders();
            break;
   case  1: ambient[0]=fl_get_slider_value(ob); break;
   case  2: ambient[1]=fl_get_slider_value(ob); break;
   case  3: ambient[2]=fl_get_slider_value(ob); break;
   case  4: diffuse[0]=fl_get_slider_value(ob); break;
   case  5: diffuse[1]=fl_get_slider_value(ob); break;
   case  6: diffuse[2]=fl_get_slider_value(ob); break;
   case  7: specular[0]=fl_get_slider_value(ob); break;
   case  8: specular[1]=fl_get_slider_value(ob); break;
   case  9: specular[2]=fl_get_slider_value(ob); break;
   case 10: position[0]=fl_get_slider_value(ob); break;
   case 11: position[1]=fl_get_slider_value(ob); break;
   case 12: position[2]=fl_get_slider_value(ob); break;
   case 13: shine=fl_get_slider_value(ob); break;
   }
   UpdateLighting();
   glutPostRedisplay();
}

void Info(FL_OBJECT *ob, long data) /* XForms callback */
{
   fl_deactivate_all_forms();
   fl_show_form(infoWindow,FL_PLACE_MOUSE,FL_TRANSIENT,"XFGL Info");
   fl_do_forms(); /* until user clicks Dismiss */
   fl_hide_form(infoWindow);
   fl_activate_all_forms();
}

void Quit(FL_OBJECT *ob, long data) /* XForms callback */
{
   exit(0);
}

void Reshape(int w,int h) /* GLUT callback */
{
   viewportWidth=w; viewportHeight=h;
   glViewport(0,0,w,h);
}

void Redraw(void) /* GLUT callback */
{
   glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
   glPushMatrix();
   glutSolidSphere(8,50,50);
   glFlush();
   glPopMatrix();
   glutSwapBuffers();
}

void XForms(void) /* GLUT callback */
{
   fl_check_forms();
}

void Click(int button,int state,int x,int y) /* GLUT callback */
{
   if (state==GLUT_DOWN) {
      int redraw=0;
      if (button==GLUT_LEFT_BUTTON) {
         position[0]=2.0*x/viewportWidth-1.0;
         position[1]=-(2.0*y/viewportHeight-1.0);
         redraw=1;
      } else if (button==GLUT_RIGHT_BUTTON) {
         position[2]=1.0*x/viewportWidth+1.0*y/viewportHeight-1.0;
         redraw=1;
      } 
      if (redraw) {
         if (position[0]<-1.0) position[0]=-1.0;
         else if (position[0]>1.0) position[0]=1.0;
         if (position[1]<-1.0) position[1]=-1.0;
         else if (position[1]>1.0) position[1]=1.0;
         if (position[2]<-1.0) position[2]=-1.0;
         else if (position[2]>1.0) position[2]=1.0;
         UpdateSliders();
         UpdateLighting();
         glutPostRedisplay();
      }
   }
}

void InitGLUT(int *argc,char *argv[])
{
   GLfloat ambient[]={0.0,0.0,0.0,1.0};
   GLfloat diffuse[]={1.0,1.0,1.0,1.0};
   GLfloat specular[]={1.0,1.0,1.0,1.0};
   GLfloat lmodelAmbient[]={0.2,0.2,0.2,1.0};
   GLfloat localView[]={0.0};

   glutInit(argc,argv);
   glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGB|GLUT_DEPTH);
   glutInitWindowSize(300,300);
   glutCreateWindow("XFGL Lighting");

   glutReshapeFunc(Reshape);
   glutDisplayFunc(Redraw);
   glutIdleFunc(XForms);
   glutMouseFunc(Click);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glOrtho(-10,10,-10,10,-10,10);
   glMatrixMode(GL_MODELVIEW);

   glLightfv(GL_LIGHT0,GL_AMBIENT,ambient);
   glLightfv(GL_LIGHT0,GL_DIFFUSE,diffuse);
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT,lmodelAmbient);
   glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER,localView);

   glFrontFace(GL_CW);
   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
   glEnable(GL_AUTO_NORMAL);
   glEnable(GL_NORMALIZE);
   glEnable(GL_DEPTH_TEST);
   glDepthFunc(GL_LESS);
}

void InitXForms(int *argc,char *argv[])
{
   fl_initialize(argc,argv,"gl",0,0);
   create_the_forms();

   fl_set_slider_return(ambientR,0);
   fl_set_slider_return(ambientG,0);
   fl_set_slider_return(ambientB,0);
   fl_set_slider_return(diffuseR,0);
   fl_set_slider_return(diffuseG,0);
   fl_set_slider_return(diffuseB,0);
   fl_set_slider_return(specularR,0);
   fl_set_slider_return(specularG,0);
   fl_set_slider_return(specularB,0);
   fl_set_slider_return(positionR,0);
   fl_set_slider_return(positionG,0);
   fl_set_slider_return(positionB,0);
   fl_set_slider_return(shininess,0);
   fl_set_slider_value(positionR,-0.5);
   fl_set_slider_bounds(positionR,-1.0,1.0);
   fl_set_slider_value(positionG,0.5);
   fl_set_slider_bounds(positionG,-1.0,1.0);
   fl_set_slider_value(positionB,0.5);
   fl_set_slider_bounds(positionB,-1.0,1.0);
   fl_show_form(controlPanel,FL_PLACE_SIZE,FL_FULLBORDER,"XFGL Control");

   Lighting(NULL,0L);
}

int main(int argc, char *argv[])
{
   InitGLUT(&argc,argv);
   InitXForms(&argc,argv);
   glutMainLoop();
   return 0;
}










