/* --------- XFGL - OpenGL Lighting / XForms Control --------- */
/* (C) 960315 -- Anneke Sicherer-Roetman -- anneke@chem.ruu.nl */
/* Note: if you find this code useful, please drop me an email */

#include <forms.h>
#include <GL/glut.h>
#include "xfgl2form.h"

#define STEP 5.0

static GLfloat Xrot=0,Xstep=STEP;
static GLfloat Yrot=0,Ystep=0;
static GLfloat Zrot=0,Zstep=0;
static GLfloat left,right,bottom,top,near,far;
static GLfloat translate;
static int viewportWidth,viewportHeight;
static int animate=0;

static void DrawAxes(void)
{
   glBegin(GL_LINES);
   glColor3f(1,0,0);
   glVertex3f(-100,0,0); glVertex3f(100,0,0);
   glColor3f(0,1,0);
   glVertex3f(0,-100,0); glVertex3f(0,100,0);
   glColor3f(0,0,1);
   glVertex3f(0,0,-100); glVertex3f(0,0,100);
   glEnd();
}

static void UpdateFrustum(void)
{
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glFrustum(left,right,bottom,top,near,far); 
   glMatrixMode(GL_MODELVIEW);
}

static void UpdateSliders(void)
{
   fl_set_slider_value(leftSlider,left);
   fl_set_slider_value(rightSlider,right);
   fl_set_slider_value(bottomSlider,bottom);
   fl_set_slider_value(topSlider,top);
   fl_set_slider_value(nearSlider,near);
   fl_set_slider_value(farSlider,far);
   fl_set_slider_value(translateSlider,translate);
}

void Frustum(FL_OBJECT *ob, long data) /* XForms callback */
{
   switch (data) {
   case  0: Xrot=Yrot=Zrot=0.0; Xstep=STEP; Ystep=Zstep=0.0;
            left=-1.0; right=1.0; bottom=-1.0; top=1.0; near=5.0; far=15.0;
            translate=-10.0; UpdateSliders(); break;
   case  1: left=fl_get_slider_value(ob); break;
   case  2: right=fl_get_slider_value(ob); break;
   case  3: bottom=fl_get_slider_value(ob); break;
   case  4: top=fl_get_slider_value(ob); break;
   case  5: near=fl_get_slider_value(ob); break;
   case  6: far=fl_get_slider_value(ob); break;
   case  7: translate=fl_get_slider_value(ob); break;
   }
   UpdateFrustum();
   glutPostRedisplay();
}

void Info(FL_OBJECT *ob, long data) /* XForms callback */
{
   fl_deactivate_all_forms();
   fl_show_form(infoWindow,FL_PLACE_MOUSE,FL_TRANSIENT,"XFGL2 Info");
   fl_do_forms(); /* until user clicks Dismiss */
   fl_hide_form(infoWindow);
   fl_activate_all_forms();
}

void Animation(FL_OBJECT *ob, long data) /* XForms callback */
{
   animate=!animate;
   if (animate) fl_set_object_label(animation,"Stop");
           else fl_set_object_label(animation,"Spin");
}

void Quit(FL_OBJECT *ob, long data) /* XForms callback */
{
   exit(0);
}

void Reshape(int w,int h) /* GLUT callback */
{
   viewportWidth=w; viewportHeight=h;
   glViewport(0,0,w,h);
}

void Redraw(void) /* GLUT callback */
{
   glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
   glPushMatrix();
   glTranslatef(0.0,0.0,translate);
/*   glRotatef(10,1,0,0); glRotatef(10,0,1,0); glRotatef(10,0,0,1); */
   DrawAxes();
   if (Xstep) 
      glRotatef(Xrot,1.0,0.0,0.0);
   else if (Ystep) 
      glRotatef(Yrot,0.0,1.0,0.0);
   else 
      glRotatef(Zrot,0.0,0.0,1.0);
   glColor3f(1,1,1);
   glutWireCube(2.0);
   glPopMatrix();
   glFlush();
   glutSwapBuffers();
}

void XForms(void) /* GLUT callback */
{
   fl_check_forms();
   if (animate) {
      Xrot+=Xstep; Yrot+=Ystep; Zrot+=Zstep;
      if (Xrot>=360.0) { 
         Xrot=Xstep=0.0; Ystep=STEP;
      } else if (Yrot>=360.0) { 
         Yrot=Ystep=0.0; Zstep=STEP; 
      } else if (Zrot>=360.0) { 
         Zrot=Zstep=0.0; Xstep=STEP; 
      }
      glutPostRedisplay();
   }
}

void InitGLUT(int *argc,char *argv[])
{
   glutInit(argc,argv);
   glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGB|GLUT_DEPTH);
   glutInitWindowSize(300,300);
   glutCreateWindow("XFGL2 Frustum");

   glutReshapeFunc(Reshape);
   glutDisplayFunc(Redraw);
   glutIdleFunc(XForms);

   UpdateFrustum();
}

void InitXForms(int *argc,char *argv[])
{
   fl_initialize(argc,argv,"xfgl2",0,0);
   create_the_forms();

#ifdef ok
   fl_set_slider_return(leftSlider,0);
   fl_set_slider_return(rightSlider,0);
   fl_set_slider_return(bottomSlider,0);
   fl_set_slider_return(topSlider,0);
   fl_set_slider_return(nearSlider,0);
   fl_set_slider_return(farSlider,0);
   fl_set_slider_return(translateSlider,0);
#endif
   fl_set_slider_value(leftSlider,-1.0);
   fl_set_slider_bounds(leftSlider,-10.0,0.0);
   fl_set_slider_value(rightSlider,1.0);
   fl_set_slider_bounds(rightSlider,0.0,10.0);
   fl_set_slider_value(bottomSlider,-1.0);
   fl_set_slider_bounds(bottomSlider,-10.0,0.0);
   fl_set_slider_value(topSlider,1.0);
   fl_set_slider_bounds(topSlider,0.0,10.0);
   fl_set_slider_value(nearSlider,5.0);
   fl_set_slider_bounds(nearSlider,0.001,10.0);
   fl_set_slider_value(farSlider,15.0);
   fl_set_slider_bounds(farSlider,10.0,20.0);
   fl_set_slider_value(translateSlider,-10.0);
   fl_set_slider_bounds(translateSlider,-20.0,0.0);
   fl_show_form(controlPanel,FL_PLACE_SIZE,FL_FULLBORDER,"XFGL2 Control");

   Frustum(NULL,0L);
}

int main(int argc, char *argv[])
{
   InitGLUT(&argc,argv);
   InitXForms(&argc,argv);
   glutMainLoop();
   return 0;
}





